

/*C*

________________________________________________________________

        Image.h
        %W% %E%, Copyright 1990, Blab, UiO
        Image processing lab, Department of Informatics
        University of Oslo
        E-mail: blab@uio.no
________________________________________________________________
  
  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted, 
  provided that this copyright notice appear in all copies and that 
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining 
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 

*/


#ifndef _Image_h
#define _Image_h

/****************************************************************
 *
 * Image Widget: Public functions
 *
 ****************************************************************/

typedef unsigned char *ImageData;


void ImageError();
int  ImageHasColormap();
int  ImagePlanes();
int  ImageGetDisplayMethod();
void ImageInstallPixelType();
int  ImageScreenXtoPos();
int  ImageScreenYtoPos();
ImageData ImageScreenXYtoAdr();
ImageData ImagePosXYtoAdr();
int  ImagePosXtoScreen();
int  ImagePosYtoScreen();
void ImageTranslateZoomPanArgs();
void ImageGetZoom();
void ImageSetZoom();
void ImageCopyImagePart();
void ImageRedisplay();
int  ImageRedisplayArea();
void ImageRedisplayScreen();
void ImageKill();


/****************************************************************
 *
 * Image format depend
 *
 ****************************************************************/

#define ImageBytePixel 		"ImageBytePixel"
#define ImageUnsignedShortPixel	"ImageUnsignedShortPixel"
#define ImageShortPixel		"ImageShortPixel"
#define ImageLongPixel		"ImageLongPixel"
#define ImageFloatPixel		"ImageFloatPixel"
#define ImageDoublePixel	"ImageDoublePixel"

char *ImageTitle();
unsigned char *ImageAddress();
int ImageWidth();
int ImageHeight();
int ImageLineLength();
int ImageXstart();
int ImageYstart();
char *ImagePixName();

/* int ImageXoffset(x)		((x) + 1)
int ImageYoffset(y)		((y) + 1)

*/


/****************************************************************
 *
 * Image widget
 *
 ****************************************************************/

/* Resources:

 Name		     Class		RepType		Default Value
 ----		     -----		-------		-------------
 background	     Background		Pixel		XtDefaultBackground
 border		     BorderColor	Pixel		XtDefaultForeground
 borderWidth	     BorderWidth	Dimension	1
 destroyCallback     Callback		Pointer		NULL
 height		     Height		Dimension	0
 mappedWhenManaged   MappedWhenManaged	Boolean		True
 sensitive	     Sensitive		Boolean		True
 width		     Width		Dimension	0
 x		     Position		Position	0
 y		     Position		Position	0

*/

/* define any special resource names here that are not in <X11/StringDefs.h> */

#define XtNimageResource		"imageResource"
#define XtCImageResource		"ImageResource"

#define XtNimageImage                   "imageImage"
#define XtCImageImage                   "ImageImage"
#define XtNimageBand                    "imageBand"
#define XtCImageBand                    "ImageBand"
#define XtNimageRedBand                 "imageRedBand"
#define XtCImageRedBand                 "ImageRedBand"
#define XtNimageGreenBand               "imageGreenBand"
#define XtCImageGreenBand               "ImageGreenBand"
#define XtNimageBlueBand                "imageBlueBand"
#define XtCImageBlueBand                "ImageBlueBand"
#define XtNimageData                    "imageData"
#define XtCImageData                    "ImageData"
#define XtNimageRedData                 "imageRedData"
#define XtCImageRedData                 "ImageRedData"
#define XtNimageGreenData               "imageGreenData"
#define XtCImageGreenData               "ImageGreenData"
#define XtNimageBlueData                "imageBlueData"
#define XtCImageBlueData                "ImageBlueData"
#define XtNimagePixtype                 "imagePixtype"
#define XtCImagePixtype                 "ImagePixtype"
#define XtNimageTitle                   "imageTitle"
#define XtCImageTitle                   "ImageTitle"
#define XtNimageWidth                   "imageWidth"
#define XtCImageWidth                   "ImageWidth"
#define XtNimageHeight                  "imageHeight"
#define XtCImageHeight                  "ImageHeight"
#define XtNimageLineLength              "imageLineLength"
#define XtCImageLineLength              "ImageLineLength"
#define XtNimageXstart                  "imageXstart"
#define XtCImageXstart                  "ImageXstart"
#define XtNimageYstart                  "imageYstart"
#define XtCImageYstart                  "ImageYstart"
#define XtNdisplayMethod              	"displayMethod"
#define XtCDisplayMethod                "DisplayMethod"
#define XtNprintValues			"printValues"
#define XtCPrintValues			"PrintValues"
#define XtNcolortab                    	"colortab"
#define XtCColortab                	"Colortab"
#define XtNfont1                	"font1"
#define XtCFont1			"Font1"
#define XtNfont2                	"font2"
#define XtCFont2			"Font2"
#define XtNfont3                	"font3"
#define XtCFont3			"Font3"
#define XtNfont4                	"font4"
#define XtCFont4			"Font4"
#ifndef XtNvisual
#define XtNvisual                	"visual"
#define XtCVisual			"Visual"
#endif
#define XtNscaletype               	"scaletype"
#define XtCScaletype               	"Scaletype"
#define XtNscalea               	"scalea"
#define XtCScalea               	"Scalea"
#define XtNscaleb               	"scaleb"
#define XtCScaleb               	"Scaleb"
#define XtNminval               	"minval"
#define XtCMinval               	"Minval"
#define XtNmaxval               	"maxval"
#define XtCMaxval               	"Maxval"
#define XtNresizeCallback		"resizeCallback"
#define XtCResizeCallback		"ResizeCallback"

/* Scale Methods   */

#define SCALE_NONE 		 0
#define SCALE_THRESHOLD 	 1
#define SCALE_LINEAR 		 2

/* Display Methods */

#define STATIC_GRAY_1	       0x0010
#define STATIC_GRAY_2          0x0020
#define STATIC_GRAY_4          0x0040
#define STATIC_GRAY_8	       0x0080
#define GRAY_SCALE_4	       0x1040
#define GRAY_SCALE_8	       0x1080
#define PSEUDO_COLOR_8         0x3080
#define PSEUDO_COLOR_8_A       0x3080
#define PSEUDO_COLOR_8_B       0x3081
#define TRUE_COLOR_24	       0x4180

/* Callback Reason */

#define ACTIVATE		10
#define RESIZE 			39

/* declare specific ImageWidget class and instance datatypes */

typedef struct _ImageClassRec*	ImageWidgetClass;
typedef struct _ImageRec*		ImageWidget;

/* declare the class constant */

extern WidgetClass imageWidgetClass;
typedef int (*resizefunc)();
typedef char * (*printffunc)();

typedef struct pixelstruct
{
  char *pixelname;
  int  pixelsize;
  resizefunc resize;
  printffunc printf;
  int txtlen;
  int scaletype;
  float scalea, scaleb;
  float minval, maxval;
} pixel;


typedef struct 
{
  int reason;
  XEvent *event;
  struct imagestruct *img;
  struct bandstruct *band;
  int x, y;
  ImageData data;
  pixel *pixtype;
  float zoom, magnification;
  int xpan, ypan;
} ImageCallbackRec, *ImageCallback; 


#endif  _Image_h


